/**
 * Test case for create.
 * Runs with mocha.
 */
'use strict'

const create = require('../lib/create.js')
const apemanport = require('apemanport')
const apemanApp = require('apeman-app')
const fs = require('fs')
const co = require('co')
const apemandb = require('apemandb')
const apemanrequest = require('apemanrequest')
const apemanmodel = require('apemanmodel')
const assert = require('assert')

let tmpDir = `${__dirname}/../tmp`
let dbFile = tmpDir + '/testing-db01.db'

const loc = require('../doc/mocks/locales')

describe('create', () => {
  let user01, sign01
  let db
  let baseUrl
  let server
  let recover01
  let request = apemanrequest.create({ jar: true })

  before(() => co(function * () {
    db = apemandb({
      env: {
        DATABASE: 'testing-database02',
        USERNAME: 'FOO',
        PASSWORD: 'BAR',
        DIALECT: 'sqlite',
        STORAGE: dbFile,
        LOGGING: false
      },
      models: `${apemanmodel.paths.SAMPLE_DEFS_DIR}/*.json`
    })
    yield db.sync({ force: true })
  }))

  before(() => co(function * () {
    const passwordSpec = 'pdkdf2:120:184'
    let port = yield apemanport.find()
    server = yield apemanApp('testing-app', {
      port,
      configuration: {
        $apps: {
          'testing-app': {
            '/': [
              require('apeman-app-json')(),
              require('apeman-app-format')(),
              require('apeman-app-send')(),
              require('apeman-app-header')(),
              require('apeman-app-form')(),
              require('apeman-app-lang')(),
              require('apeman-app-locale')(loc),
              require('apeman-app-link')(),
              require('apeman-app-jsonapi')(),
              require('apeman-app-session')(
                db.model('Session'), '1234qwer'
              ),
              require('apeman-app-signed')(),
              require('apeman-app-rest')(
                db.model('User'),
                {
                  pathname: '/api/users'
                }
              ),
              require('apeman-app-sign')(
                db.model('UserSign'),
                {
                  passwordSpec,
                  pathname: '/api/sign'
                }
              ),
              create(
                db.model('UserRecover'),
                {
                  passwordSpec,
                  pathname: '/api/recover'
                })
            ]
          }
        }
      }
    })
    baseUrl = `http://localhost:${port}`
  }))

  after(() => co(function * () {
    yield server.close()
    yield new Promise((resolve) =>
      setTimeout(() => {
        fs.unlink(dbFile, () => {
          resolve()
        })
      }, 100) // Wait for flush.
    )
  }))

  it('Create user', () => co(function * () {
    let res = yield request({
      method: 'POST',
      url: `${baseUrl}/api/users`,
      json: true,
      form: {
        data: {
          type: 'users',
          attributes: {
            key: 'foo_bar',
            email: 'user01@exampole.com'
          }
        }
      }
    })
    let { body } = res
    assert.equal(res.statusCode, 201)
    let { data } = body
    user01 = Object.assign(
      {},
      data.attributes,
      { id: data.id }
    )
    assert.equal(user01.key, 'foo_bar')
    assert.equal(user01.email, 'user01@exampole.com')
  }))

  it('Create sign', () => co(function * () {
    let res = yield request({
      method: 'POST',
      url: `${baseUrl}/api/sign`,
      json: true,
      form: {
        data: {
          type: 'user_signs',
          attributes: {
            key: user01.key,
            password: 'the-password-of-user01',
            kind: 'user.key'
          },
          relationships: {
            user: {
              data: {
                type: 'users',
                id: user01.id
              }
            }
          }
        }
      }
    })
    let { body } = res
    assert.equal(res.statusCode, 201)
    let { data } = body
    assert.ok(data.attributes.signupAt)
    assert.ok(!data.attributes.hash)
    assert.ok(!data.attributes.hashSalt)
    sign01 = Object.assign({ id: data.id }, data.attributes)
    assert.ok(sign01)
  }))

  it('Create fail.', () => co(function * () {
    let res = yield request({
      url: `${baseUrl}/api/recover/`,
      method: 'POST',
      json: true,
      form: {
        data: {
          type: 'user_recovers',
          attributes: {
            email: '__invalid_email__'
          }
        }
      }
    })
    let { body } = res
    assert.equal(res.statusCode, 404)
    assert.ok(body)
  }))

  it('Create success.', () => co(function * () {
    let res = yield request({
      url: `${baseUrl}/api/recover/`,
      method: 'POST',
      json: true,
      form: {
        data: {
          type: 'user_recovers',
          attributes: {
            email: 'user01@exampole.com'
          }
        }
      }
    })
    let { body } = res
    assert.equal(res.statusCode, 200)
    let { data } = body
    recover01 = Object.assign({ id: data.id }, data.attributes)
    assert.ok(recover01.token)
  }))

  it('Update fail.', () => co(function * () {
    let res = yield request({
      url: `${baseUrl}/api/recover/${recover01.uuid}`,
      method: 'PATCH',
      json: true,
      form: {
        data: {
          type: 'user_recovers',
          attributes: {
            token: '__invalid_token__',
            password: 'hoge'
          }
        }
      }
    })
    let { body } = res
    assert.equal(res.statusCode, 401)
    assert.ok(body.errors)
    assert.equal(body.errors[ 0 ].meta.trigger, 'apeman-app-recover')
  }))

  it('Update success.', () => co(function * () {
    let res = yield request({
      url: `${baseUrl}/api/recover/${recover01.uuid}`,
      method: 'PATCH',
      json: true,
      form: {
        data: {
          type: 'user_recovers',
          attributes: {
            token: recover01.token,
            password: 'new_password'
          }
        }
      }
    })
    let { body } = res
    assert.equal(res.statusCode, 200)
    let { data } = body
    assert.equal(data.type, 'user_recovers')
    assert.equal(data.attributes.expired, false)
    assert.ok(data.attributes.token)
    assert.ok(data.attributes.done)
  }))

  it('Update with done token.', () => co(function * () {
    let res = yield request({
      url: `${baseUrl}/api/recover/${recover01.uuid}`,
      method: 'PATCH',
      json: true,
      form: {
        data: {
          type: 'user_recovers',
          attributes: {
            token: recover01.token,
            password: 'new_password'
          }
        }
      }
    })
    let { body } = res
    assert.equal(res.statusCode, 401)
    let errors = body.errors
    assert.equal(errors.length, 1)
  }))

  it('Signin with new password', () => co(function * () {
    let res = yield request({
      url: `${baseUrl}/api/sign/pass`,
      method: 'POST',
      json: true,
      form: {
        data: {
          type: 'user_sign_passes',
          attributes: {
            key: user01.key,
            password: 'new_password'
          }
        }
      }
    })
    let { body } = res
    assert.equal(res.statusCode, 200)
    assert.ok(!body.errors)
    assert.ok(body.data)
  }))
})

/* global describe, before, after, it */
